import copy
from pathlib import Path

import pytest
from pytest import MonkeyPatch

from .gradle_properties import GradlePropertiesMetadataHook


@pytest.mark.parametrize(
    # fmt: off
    "package, op,key,py_version,gradle_version,rc_upper_bound,full_version",
    [
        ("P", "~=", "KEY", "4",       "1.2.3",   False, "P~=1.2.3.4"),
        ("P", "~=", "KEY", "4.5",     "1.2.3",   False, "P~=1.2.3.4.5"),
        ("P", ">=", "KEY", "4.5",     "1.2.3",   False, "P>=1.2.3.4.5"),
        ("P", "~=", "KEY", "4.5",     "1.2.3-6", False, "P~=1.2.3.4.5rc6"),
        ("P", "~=", "KEY", "4.5",     "1.2.3-6", True,  "P~=1.2.3.4.5rc6,<1.2.3.4.5rc7"),
        ("P", "~=", "KEY", "4.5dev6", "1.2.3",   False, "P~=1.2.3.4.5.dev6"),
        ("P", "~=", "KEY", "4.5dev7", "1.2.3-6", False, "P~=1.2.3.4.5rc6.dev7"),
        ("P", "~=", "KEY", "4.5dev8", "1.2.3-6", True,  "P~=1.2.3.4.5rc6.dev8,<1.2.3.4.5rc7.dev8"),
    ],
    # fmt: on
)
def test_gradle_properties_deps(
    tmp_path: Path,
    package: str,
    op: str,
    key: str,
    py_version: str,
    gradle_version: str,
    rc_upper_bound: bool,
    full_version: str,
    monkeypatch: MonkeyPatch,
):
    # arrange
    monkeypatch.setenv("HATCH_GRADLE_DIR", "gradle_dir")

    gradle_properties = tmp_path / "gradle_dir" / "gradle.properties"
    gradle_properties.parent.mkdir()
    gradle_properties.write_text(f"{key}={gradle_version}")

    hook = GradlePropertiesMetadataHook(
        tmp_path.as_posix(),
        {
            "dependencies": [
                {
                    "package": package,
                    "op": op,
                    "key": key,
                    "py-version": py_version,
                    "rc-upper-bound": rc_upper_bound,
                }
            ],
        },
    )

    orig_metadata = {
        "dynamic": [
            "dependencies",
            "optional-dependencies",
        ],
    }

    # act
    metadata = copy.deepcopy(orig_metadata)
    hook.update(metadata)

    # assert
    assert metadata == orig_metadata | {
        "dependencies": [full_version],
        "optional-dependencies": {},
    }



import json
from typing import Annotated, Any

from pydantic import Field, field_validator

from hatch_gradle_version.common.gradle import GradleVersion
from hatch_gradle_version.common.model import GradlePath

from .base import BaseVersionSource


class JSONVersionSource(BaseVersionSource):
    PLUGIN_NAME = "json"

    json_path: GradlePath
    key: Annotated[list[str], Field(min_length=1)]

    def get_gradle_version(self) -> GradleVersion:
        with self.json_path.open() as f:
            data = json.load(f)

        raw_version = data
        for key in self.key:
            raw_version = raw_version[key]

        return GradleVersion.from_raw(raw_version, {})

    @field_validator("key", mode="before")
    @classmethod
    def _split_key(cls, value: Any):
        match value:
            case str():
                return value.split(".")
            case _:
                return value



from pathlib import Path
from textwrap import dedent

from pytest import MonkeyPatch

from hatch_gradle_version.common.gradle import GradleVersion

from .json import JSONVersionSource


def test_json_version(tmp_path: Path, monkeypatch: MonkeyPatch):
    # arrange
    monkeypatch.setenv("HATCH_GRADLE_DIR", "gradle_dir")

    json = tmp_path / "gradle_dir" / "mod.json"
    json.parent.mkdir()
    json.write_text(
        dedent(
            """
            {
                "core": {
                    "name": "hexbound",
                    "version": "0.1.4+1.19.2",
                    "repository": "https://github.com/Cypher121/hexbound/"
                },
                "platforms": {
                    "modrinth": {
                        "id": "PHgo4bVw"
                    },
                    "curseforge": {
                        "id": "739791"
                    }
                }
            }
            """
        )
    )

    hook = JSONVersionSource.from_config(
        tmp_path.as_posix(),
        {
            "source": "json",
            "py-path": "",
            "json-path": "mod.json",
            "key": "core.version",
        },
    )

    gradle_version = hook.get_gradle_version()

    assert gradle_version == GradleVersion(
        raw_version="0.1.4+1.19.2",
        version="0.1.4",
        rc=None,
        build="1.19.2",
        extra_versions={},
    )



import tomllib
from functools import cached_property
from pathlib import Path

from hatch_gradle_version.common.gradle import GradleDependency, GradleVersion
from hatch_gradle_version.common.model import GradlePath

from .base import BaseMetadataHook


class VersionCatalogMetadataHook(BaseMetadataHook):
    PLUGIN_NAME = "version-catalog"

    path: GradlePath = Path("gradle/libs.versions.toml")

    def parse_gradle_dependency(self, dependency: GradleDependency):
        raw_version = self.versions[dependency.key]
        gradle_version = GradleVersion.from_raw(raw_version, {})
        return dependency.version_specifier(gradle_version)

    @cached_property
    def versions(self):
        with self.path.open("rb") as f:
            version_catalog = tomllib.load(f)
        return version_catalog["versions"]
